@@ -1,11 +1,12 @@
 
 /* pngwio.c - functions for data output
  *
- * libpng 1.00.97
+ * libpng 1.0.0
  * For conditions of distribution and use, see copyright notice in png.h
  * Copyright (c) 1995, 1996 Guy Eric Schalnat, Group 42, Inc.
  * Copyright (c) 1996, 1997 Andreas Dilger
- * May 28, 1997
+ * Copyright (c) 1998, Glenn Randers-Pehrson
+ * March 8, 1998
  *
  * This file provides a location for all output.  Users which need
  * special handling are expected to write functions which have the same
@@ -33,6 +34,7 @@ png_write_data(png_structp png_ptr, png_bytep data, png_size_t length)
       png_error(png_ptr, "Call to NULL write function");
 }
 
+#if !defined(PNG_NO_STDIO)
 /* This is the function which does the actual writing of data.  If you are
    not writing to a standard C stream, you should create a replacement
    write_data function and use it at run time with png_set_write_fn(), rather
@@ -98,6 +100,7 @@ png_default_write_data(png_structp png_ptr, png_bytep data, png_size_t length)
    }
 }
 
+#endif
 #endif
 
 /* This function is called to output any data pending writing (normally
@@ -111,6 +114,7 @@ png_flush(png_structp png_ptr)
       (*(png_ptr->output_flush_fn))(png_ptr);
 }
 
+#if !defined(PNG_NO_STDIO)
 static void
 png_default_flush(png_structp png_ptr)
 {
@@ -120,6 +124,7 @@ png_default_flush(png_structp png_ptr)
       fflush(io_ptr);
 }
 #endif
+#endif
 
 /* This function allows the application to supply new output functions for
    libpng if standard C streams aren't being used.
@@ -149,16 +154,24 @@ png_set_write_fn(png_structp png_ptr, png_voidp io_ptr,
 {
    png_ptr->io_ptr = io_ptr;
 
+#if !defined(PNG_NO_STDIO)
    if (write_data_fn != NULL)
       png_ptr->write_data_fn = write_data_fn;
    else
       png_ptr->write_data_fn = png_default_write_data;
+#else
+   png_ptr->write_data_fn = write_data_fn;
+#endif
 
 #if defined(PNG_WRITE_FLUSH_SUPPORTED)
+#if !defined(PNG_NO_STDIO)
    if (output_flush_fn != NULL)
       png_ptr->output_flush_fn = output_flush_fn;
    else
       png_ptr->output_flush_fn = png_default_flush;
+#else
+   png_ptr->output_flush_fn = output_flush_fn;
+#endif
 #endif /* PNG_WRITE_FLUSH_SUPPORTED */
 
    /* It is an error to read while writing a png file */
@@ -167,7 +180,7 @@ png_set_write_fn(png_structp png_ptr, png_voidp io_ptr,
 
 #if defined(USE_FAR_KEYWORD) 
 #if defined(_MSC_VER)   
-void *far_to_near(png_structp png_ptr,png_voidp ptr, int check)
+void *png_far_to_near(png_structp png_ptr,png_voidp ptr, int check)
 {
    void *near_ptr;   
    void FAR *far_ptr;
@@ -179,7 +192,7 @@ void *far_to_near(png_structp png_ptr,png_voidp ptr, int check)
    return(near_ptr);
 }
 #  else
-void *far_to_near(png_structp png_ptr,png_voidp ptr, int check)
+void *png_far_to_near(png_structp png_ptr,png_voidp ptr, int check)
 {
    void *near_ptr;   
    void FAR *far_ptr;
